@@ -15,7 +15,6 @@
 import os
 import sys
 import threading
-import time
 
 import httplib2
 import netaddr
@@ -72,8 +71,7 @@ class MonitorDaemon(daemon.Daemon):
         self._thread_ip_monitor.start()
         self._thread_read_queue.start()
         self.handle_initial_state()
-        while True:
-            time.sleep(1)
+        self._thread_read_queue.join()
 
     def read_queue(self, _queue, event_stop, event_started):
         event_started.wait()
@@ -106,12 +104,12 @@ class MonitorDaemon(daemon.Daemon):
             for address in ip.addr.list():
                 if address.get('cidr') == self.cidr:
                     state = 'master'
-                    self.write_state_change(state)
-                    self.notify_agent(state)
                     break
 
             LOG.debug('Initial status of router %s is %s',
                       self.router_id, state)
+            self.write_state_change(state)
+            self.notify_agent(state)
         except Exception:
             LOG.exception('Failed to get initial status of router %s',
                           self.router_id)
