@@ -15,14 +15,6 @@
 import collections
 import re
 
-from neutron.agent.linux import external_process
-from neutron.agent.linux import ip_lib
-from neutron.agent.ovn.metadata import driver as metadata_driver
-from neutron.agent.ovn.metadata import ovsdb
-from neutron.agent.ovn.metadata import server as metadata_server
-from neutron.common.ovn import constants as ovn_const
-from neutron.common import utils
-from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as config
 from neutron_lib import constants as n_const
 from oslo_concurrency import lockutils
 from oslo_log import log
@@ -30,6 +22,17 @@ from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import vlog
 import six
+import tenacity
+
+from neutron.agent.linux import external_process
+from neutron.agent.linux import ip_lib
+from neutron.agent.linux import iptables_manager
+from neutron.agent.ovn.metadata import driver as metadata_driver
+from neutron.agent.ovn.metadata import ovsdb
+from neutron.agent.ovn.metadata import server as metadata_server
+from neutron.common.ovn import constants as ovn_const
+from neutron.common import utils
+from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as config
 
 
 LOG = log.getLogger(__name__)
@@ -230,6 +233,10 @@ class MetadataAgent(object):
 
         proxy.wait()
 
+    @tenacity.retry(
+        wait=tenacity.wait_exponential(
+            max=config.get_ovn_ovsdb_retry_max_interval()),
+        reraise=True)
     def register_metadata_agent(self):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
@@ -343,6 +350,24 @@ class MetadataAgent(object):
         else:
             self.teardown_datapath(datapath)
 
+    def _ensure_datapath_checksum(self, namespace):
+        """Ensure the correct checksum in the metadata packets in DPDK bridges
+
+        (LP#1904871) In DPDK deployments (integration bridge datapath_type ==
+        "netdev"), the checksum between the metadata namespace and OVS is not
+        correctly populated.
+        """
+        if (self.ovs_idl.db_get(
+                'Bridge', self.ovn_bridge, 'datapath_type').execute() !=
+                ovn_const.CHASSIS_DATAPATH_NETDEV):
+            return
+
+        iptables_mgr = iptables_manager.IptablesManager(
+            use_ipv6=True, nat=False, namespace=namespace, external_lock=False)
+        rule = '-p tcp -m tcp -j CHECKSUM --checksum-fill'
+        iptables_mgr.ipv4['mangle'].add_rule('POSTROUTING', rule, wrap=False)
+        iptables_mgr.apply()
+
     def provision_datapath(self, datapath):
         """Provision the datapath so that it can serve metadata.
 
@@ -450,6 +475,9 @@ class MetadataAgent(object):
             'Interface', veth_name[0],
             ('external_ids', {'iface-id': port.logical_port})).execute()
 
+        # Ensure the correct checksum in the metadata traffic.
+        self._ensure_datapath_checksum(namespace)
+
         # Spawn metadata proxy if it's not already running.
         metadata_driver.MetadataDriver.spawn_monitored_metadata_proxy(
             self._process_monitor, namespace, ovn_const.METADATA_PORT,
