@@ -875,9 +875,14 @@ class Timer(object):
 
     The timeout exception can be suppressed; when the time expires, the context
     finishes without rising TimerTimeout.
+
+    NOTE(ralonsoh): this class, when a timeout is defined, cannot be used in
+    other than the main thread. When a timeout is defined, an alarm signal is
+    set. Only the main thread is allowed to set a signal handler and the signal
+    handlers are always executed in this main thread [1].
+    [1] https://docs.python.org/3/library/signal.html#signals-and-threads
     """
     def __init__(self, timeout=None, raise_exception=True):
-        super(Timer, self).__init__()
         self.start = self.delta = None
         self._timeout = int(timeout) if timeout else None
         self._timeout_flag = False
@@ -926,7 +931,7 @@ class Timer(object):
         return (datetime.datetime.now() - self.start).total_seconds()
 
 
-def _collect_profiler_info():
+def collect_profiler_info():
     p = profiler.get()
     if p:
         return {
@@ -945,7 +950,7 @@ def spawn(func, *args, **kwargs):
     but we re-initialize osprofiler in threads spawn()-ed.
     """
 
-    profiler_info = _collect_profiler_info()
+    profiler_info = collect_profiler_info()
 
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
@@ -959,7 +964,7 @@ def spawn(func, *args, **kwargs):
 def spawn_n(func, *args, **kwargs):
     """See spawn() above"""
 
-    profiler_info = _collect_profiler_info()
+    profiler_info = collect_profiler_info()
 
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
@@ -982,3 +987,15 @@ def timecost(f):
         ret = f(*args, **kwargs)
         return ret
     return wrapper
+
+
+class SingletonDecorator(object):
+
+    def __init__(self, klass):
+        self._klass = klass
+        self._instance = None
+
+    def __call__(self, *args, **kwargs):
+        if self._instance is None:
+            self._instance = self._klass(*args, **kwargs)
+        return self._instance
