@@ -42,7 +42,6 @@ import six
 
 from neutron.agent.common import utils
 from neutron.api.rpc.callbacks import version_manager
-from neutron.common import constants as n_const
 from neutron.conf.agent.database import agents_db
 from neutron.db.models import agent as agent_model
 from neutron.extensions import _availability_zone_filter_lib as azfil_ext
@@ -357,18 +356,21 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         """
         return candidate_hosts
 
-    def _log_heartbeat(self, state, agent_db, agent_conf):
+    def _log_heartbeat(self, state, agent_db, agent_conf, agent_timestamp):
         if agent_conf.get('log_agent_heartbeats'):
             delta = timeutils.utcnow() - agent_db.heartbeat_timestamp
             LOG.info("Heartbeat received from %(type)s agent on "
-                     "host %(host)s, uuid %(uuid)s after %(delta)s",
+                     "host %(host)s, uuid %(uuid)s after %(delta)s, sent at "
+                     "%(agent_timestamp)s",
                      {'type': agent_db.agent_type,
                       'host': agent_db.host,
                       'uuid': state.get('uuid'),
-                      'delta': delta})
+                      'delta': delta,
+                      'agent_timestamp': agent_timestamp})
 
     @db_api.retry_if_session_inactive()
-    def create_or_update_agent(self, context, agent_state):
+    def create_or_update_agent(self, context, agent_state,
+                               agent_timestamp=None):
         """Registers new agent in the database or updates existing.
 
         Returns tuple of agent status and state.
@@ -406,7 +408,8 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 if agent_state.get('start_flag'):
                     res['started_at'] = current_time
                 greenthread.sleep(0)
-                self._log_heartbeat(agent_state, agent, configurations_dict)
+                self._log_heartbeat(agent_state, agent, configurations_dict,
+                                    agent_timestamp)
                 agent.update_fields(res)
                 agent.update()
                 event_type = events.AFTER_UPDATE
@@ -421,12 +424,14 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 greenthread.sleep(0)
                 agent.create()
                 event_type = events.AFTER_CREATE
-                self._log_heartbeat(agent_state, agent, configurations_dict)
+                self._log_heartbeat(agent_state, agent, configurations_dict,
+                                    agent_timestamp)
                 status = agent_consts.AGENT_NEW
             greenthread.sleep(0)
 
         agent_state['agent_status'] = status
         agent_state['admin_state_up'] = agent.admin_state_up
+        agent_state['id'] = agent.id
         registry.publish(resources.AGENT, event_type, self,
                          payload=events.DBEventPayload(
                              context=context, metadata={
@@ -477,7 +482,7 @@ class AgentExtRpcCallback(object):
     """
 
     target = oslo_messaging.Target(version='1.2',
-                                   namespace=n_const.RPC_NAMESPACE_STATE)
+                                   namespace=constants.RPC_NAMESPACE_STATE)
     START_TIME = timeutils.utcnow()
 
     def __init__(self, plugin=None):
@@ -514,7 +519,7 @@ class AgentExtRpcCallback(object):
         if not self.plugin:
             self.plugin = directory.get_plugin()
         agent_status, agent_state = self.plugin.create_or_update_agent(
-            context, agent_state)
+            context, agent_state, time)
         self._update_local_agent_resource_versions(context, agent_state)
         return agent_status
 
