@@ -207,12 +207,13 @@ class DbBasePluginCommon(object):
                'max_prefixlen': max_prefixlen,
                'is_default': subnetpool['is_default'],
                'shared': subnetpool['shared'],
-               'prefixes': [prefix.cidr for prefix in subnetpool['prefixes']],
+               'prefixes': [str(prefix.cidr)
+                            for prefix in subnetpool['prefixes']],
                'ip_version': subnetpool['ip_version'],
                'default_quota': subnetpool['default_quota'],
                'address_scope_id': subnetpool['address_scope_id']}
         resource_extend.apply_funcs(
-            subnetpool_def.COLLECTION_NAME, res, subnetpool)
+            subnetpool_def.COLLECTION_NAME, res, subnetpool.db_obj)
         return db_utils.resource_fields(res, fields)
 
     def _make_port_dict(self, port, fields=None,
@@ -263,11 +264,6 @@ class DbBasePluginCommon(object):
             raise exceptions.SubnetNotFound(subnet_id=id)
         return subnet
 
-    def _network_exists(self, context, network_id):
-        query = model_query.query_with_hooks(
-            context, models_v2.Network, field='id')
-        return query.filter(models_v2.Network.id == network_id).first()
-
     def _get_subnet_object(self, context, id):
         subnet = subnet_obj.Subnet.get_object(context, id=id)
         if not subnet:
@@ -306,21 +302,17 @@ class DbBasePluginCommon(object):
         return subnet_obj.Subnet.get_objects(context,
                                              subnetpool_id=subnetpool_id)
 
-    def _get_subnets(self, context, filters=None, fields=None,
-                     sorts=None, limit=None, marker=None,
-                     page_reverse=False):
+    def _get_subnets(self, context, filters=None, sorts=None, limit=None,
+                     marker=None, page_reverse=False):
         pager = base_obj.Pager(sorts, limit, page_reverse, marker)
         filters = filters or {}
         # turn the CIDRs into a proper subnets
         if filters.get('cidr'):
             filters.update(
                 {'cidr': [netaddr.IPNetwork(x).cidr for x in filters['cidr']]})
-        # TODO(ihrachys) remove explicit reader usage when subnet OVO switches
-        # to engine facade by default
-        with db_api.CONTEXT_READER.using(context):
-            return subnet_obj.Subnet.get_objects(context, _pager=pager,
-                                                 validate_filters=False,
-                                                 **filters)
+        return subnet_obj.Subnet.get_objects(context, _pager=pager,
+                                             validate_filters=False,
+                                             **filters)
 
     def _make_network_dict(self, network, fields=None,
                            process_extensions=True, context=None):
