@@ -56,10 +56,14 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
     @staticmethod
     def _gateway_ip_str(subnet, cidr_net):
         if subnet.get('gateway_ip') is const.ATTR_NOT_SPECIFIED:
-            if subnet.get('version') == const.IP_VERSION_6:
-                return str(netaddr.IPNetwork(cidr_net).network)
+            if subnet.get('ip_version') == const.IP_VERSION_6:
+                gateway_ip = netaddr.IPNetwork(cidr_net).network
+                pd_net = netaddr.IPNetwork(const.PROVISIONAL_IPV6_PD_PREFIX)
+                if gateway_ip == pd_net.network:
+                    return
             else:
-                return str(netaddr.IPNetwork(cidr_net).network + 1)
+                gateway_ip = netaddr.IPNetwork(cidr_net).network + 1
+            return str(gateway_ip)
         return subnet.get('gateway_ip')
 
     @staticmethod
@@ -658,6 +662,12 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
             context, network_id, host, service_type, fixed_configured,
             fixed_ips, distributed_service=distributed_service)
         if subnets:
+            msg = ('This subnet is being modified by another concurrent '
+                   'operation')
+            for subnet in subnets:
+                subnet.lock_register(
+                    context, exc.SubnetInUse(subnet_id=subnet.id, reason=msg),
+                    id=subnet.id)
             subnet_dicts = [self._make_subnet_dict(subnet, context=context)
                             for subnet in subnets]
             # Give priority to subnets with service_types
