@@ -29,10 +29,10 @@ from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import agent as agent_exc
 from neutron_lib.exceptions import l3 as l3_exc
+from neutron_lib.objects import exceptions as o_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as plugin_utils
-from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import helpers as log_helper
 from oslo_log import log as logging
@@ -375,6 +375,27 @@ class DVRResourceOperationHandler(object):
             # to clean up the fip namespace as it is no longer required.
             self.l3plugin.l3_rpc_notifier.delete_fipnamespace_for_ext_net(
                 payload.context, network_id)
+            # Delete the Floating IP agent gateway port
+            # bindings on all hosts
+            l3_obj.DvrFipGatewayPortAgentBinding.delete_objects(
+                payload.context,
+                network_id=network_id)
+
+    def _delete_fip_agent_port(self, context, network_id, host_id):
+        try:
+            l3_agent_db = self._get_agent_by_type_and_host(
+                context, const.AGENT_TYPE_L3, host_id)
+        except agent_exc.AgentNotFoundByTypeHost:
+            LOG.warning("%(ag)s agent not found for the given host: %(host)s",
+                        {'ag': const.AGENT_TYPE_L3,
+                         'host': host_id})
+            return
+        try:
+            l3_obj.DvrFipGatewayPortAgentBinding(
+                context, network_id=network_id,
+                agent_id=l3_agent_db['id']).delete()
+        except n_exc.ObjectNotFound:
+            pass
 
     def delete_floatingip_agent_gateway_port(self, context, host_id,
                                              ext_net_id):
@@ -387,6 +408,8 @@ class DVRResourceOperationHandler(object):
         for p in ports:
             if not host_id or p[portbindings.HOST_ID] == host_id:
                 self._core_plugin.ipam.delete_port(context, p['id'])
+                self._delete_fip_agent_port(
+                    context, ext_net_id, p[portbindings.HOST_ID])
                 if host_id:
                     return
 
@@ -1035,9 +1058,13 @@ class _DVRAgentInterfaceMixin(object):
 
     def create_fip_agent_gw_port_if_not_exists(self, context, network_id,
                                                host):
-        # TODO(slaweq): add proper constraint on database level to avoid
-        # creation of duplicated Floating IP gateway ports for same network and
-        # same L3 agent. When this will be done, we can get rid of this lock.
+        """Function to return the FIP Agent GW port.
+
+        This function will create a FIP Agent GW port
+        if required. If the port already exists, it
+        will return the existing port and will not
+        create a new one.
+        """
         try:
             l3_agent_db = self._get_agent_by_type_and_host(
                 context, const.AGENT_TYPE_L3, host)
@@ -1046,48 +1073,58 @@ class _DVRAgentInterfaceMixin(object):
                         {'ag': const.AGENT_TYPE_L3,
                          'host': host})
             return
+        if not l3_agent_db:
+            return
+
         l3_agent_mode = self._get_agent_mode(l3_agent_db)
         if l3_agent_mode == const.L3_AGENT_MODE_DVR_NO_EXTERNAL:
             return
-        if not l3_agent_db:
-            return
-        lock_name = 'fip-gw-lock-' + network_id + '-' + host
-        with lockutils.lock(lock_name, external=True):
-            return self._create_fip_agent_gw_port_if_not_exists(
-                context, network_id, host, l3_agent_db)
 
-    def _create_fip_agent_gw_port_if_not_exists(self, context, network_id,
-                                                host, l3_agent_db):
-        """Function to return the FIP Agent GW port.
-
-        This function will create a FIP Agent GW port
-        if required. If the port already exists, it
-        will return the existing port and will not
-        create a new one.
-        """
         LOG.debug("Agent ID exists: %s", l3_agent_db['id'])
         agent_port = self._get_agent_gw_ports_exist_for_network(
             context, network_id, host, l3_agent_db['id'])
         if not agent_port:
-            LOG.info("Floating IP Agent Gateway port for network %s "
-                     "does not exist on host %s. Creating one.",
-                     network_id, host)
-            port_data = {'tenant_id': '',
-                         'network_id': network_id,
-                         'device_id': l3_agent_db['id'],
-                         'device_owner': const.DEVICE_OWNER_AGENT_GW,
-                         portbindings.HOST_ID: host,
-                         'admin_state_up': True,
-                         'name': ''}
-            agent_port = plugin_utils.create_port(
-                self._core_plugin, context, {'port': port_data})
-            if not agent_port:
-                msg = _("Unable to create Floating IP Agent Gateway port")
-                raise n_exc.BadRequest(resource='router', msg=msg)
-            LOG.debug("Floating IP Agent Gateway port %(gw)s created "
-                      "for the destination host: %(dest_host)s",
-                      {'gw': agent_port,
-                       'dest_host': host})
+            LOG.info("Floating IP Agent Gateway port for network %(network)s "
+                     "does not exist on host %(host)s. Creating one.",
+                     {'network': network_id,
+                      'host': host})
+            fip_agent_port_obj = l3_obj.DvrFipGatewayPortAgentBinding(
+                context,
+                network_id=network_id,
+                agent_id=l3_agent_db['id']
+            )
+            try:
+                fip_agent_port_obj.create()
+            except o_exc.NeutronDbObjectDuplicateEntry:
+                LOG.debug("Floating IP Agent Gateway port for network "
+                          "%(network)s already exists on host %(host)s. "
+                          "Probably it was just created by other worker.",
+                          {'network': network_id,
+                           'host': host})
+                agent_port = self._get_agent_gw_ports_exist_for_network(
+                    context, network_id, host, l3_agent_db['id'])
+                LOG.debug("Floating IP Agent Gateway port %(gw)s found "
+                          "for the destination host: %(dest_host)s",
+                          {'gw': agent_port,
+                           'dest_host': host})
+            else:
+                port_data = {'tenant_id': '',
+                             'network_id': network_id,
+                             'device_id': l3_agent_db['id'],
+                             'device_owner': const.DEVICE_OWNER_AGENT_GW,
+                             portbindings.HOST_ID: host,
+                             'admin_state_up': True,
+                             'name': ''}
+                agent_port = plugin_utils.create_port(
+                    self._core_plugin, context, {'port': port_data})
+                if not agent_port:
+                    fip_agent_port_obj.delete()
+                    msg = _("Unable to create Floating IP Agent Gateway port")
+                    raise n_exc.BadRequest(resource='router', msg=msg)
+                LOG.debug("Floating IP Agent Gateway port %(gw)s created "
+                          "for the destination host: %(dest_host)s",
+                          {'gw': agent_port,
+                           'dest_host': host})
 
         self._populate_mtu_and_subnets_for_ports(context, [agent_port])
         return agent_port
