@@ -54,7 +54,7 @@ class TestMonitorDaemon(base.BaseLoggingTestCase):
 
         self._generate_cmd_opts()
         self.ext_process = external_process.ProcessManager(
-            None, self.router_id, namespace=self.router.namespace,
+            conf=None, uuid=self.router_id, namespace=self.router.namespace,
             service='test_ip_mon', pids_path=self.conf_dir,
             default_cmd_callback=self._callback, run_as_root=True,
             pid_file=self.pid_file)
@@ -99,11 +99,15 @@ class TestMonitorDaemon(base.BaseLoggingTestCase):
         try:
             utils.wait_until_true(text_in_file, timeout=15)
         except utils.WaitTimeout:
+            devices = {}
+            for dev in ip_lib.IPWrapper(
+                    namespace=self.router.namespace).get_devices():
+                devices[dev.name] = [addr['cidr'] for addr in dev.addr.list()]
             # NOTE: we need to read here the content of the file.
-            raise RuntimeError(
-                'Text not found in file %(file_name)s: "%(text)s". File '
-                'content: %(file_content)s' %
-                {'file_name': file_name, 'text': text,
+            self.fail(
+                'Text not found in file %(file_name)s: "%(text)s".\nDevice '
+                'addresses: %(devices)s.\nFile content:\n%(file_content)s' %
+                {'file_name': file_name, 'text': text, 'devices': devices,
                  'file_content': open(file_name).read()})
 
     def test_new_fip_sends_garp(self):
