@@ -23,6 +23,7 @@ import testtools
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.common import utils
+from neutron.agent import firewall as agent_firewall
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import firewall as ovsfw
@@ -39,8 +40,14 @@ TESTING_SEGMENT = 1000
 
 def create_ofport(port_dict, network_type=None,
                   physical_network=None, segment_id=TESTING_SEGMENT):
+    allowed_pairs_v4 = ovsfw.OFPort._get_allowed_pairs(
+        port_dict, version=constants.IPv4)
+    allowed_pairs_v6 = ovsfw.OFPort._get_allowed_pairs(
+        port_dict, version=constants.IPv6)
     ovs_port = mock.Mock(vif_mac='00:00:00:00:00:00', ofport=1,
-                         port_name="port-name")
+                         port_name="port-name",
+                         allowed_pairs_v4=allowed_pairs_v4,
+                         allowed_pairs_v6=allowed_pairs_v6)
     return ovsfw.OFPort(port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG,
                         segment_id=segment_id,
                         network_type=network_type,
@@ -439,6 +446,9 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             ovs_lib, 'OVSBridge', autospec=True).start()
         securitygroups_rpc.register_securitygroups_opts()
         self.firewall = ovsfw.OVSFirewallDriver(mock_bridge)
+        self.delete_invalid_conntrack_entries_mock = mock.patch.object(
+            self.firewall.ipconntrack,
+            "delete_conntrack_state_by_remote_ips").start()
         self.mock_bridge = self.firewall.int_br
         self.mock_bridge.reset_mock()
         self.fake_ovs_port = FakeOVSPort('port', 1, '00:00:00:00:00:00')
@@ -463,6 +473,16 @@ class TestOVSFirewallDriver(base.BaseTestCase):
              'direction': constants.EGRESS_DIRECTION}]
         self.firewall.update_security_group_rules(2, security_group_rules)
 
+    def _assert_invalid_conntrack_entries_deleted(self, port_dict):
+        port_dict['of_port'] = mock.Mock(vlan_tag=10)
+        self.delete_invalid_conntrack_entries_mock.assert_has_calls([
+            mock.call(
+                [port_dict], constants.IPv4, set(),
+                mark=ovsfw_consts.CT_MARK_INVALID),
+            mock.call(
+                [port_dict], constants.IPv6, set(),
+                mark=ovsfw_consts.CT_MARK_INVALID)])
+
     @property
     def port_ofport(self):
         return self.mock_bridge.br.get_vif_port_by_id.return_value.ofport
@@ -620,6 +640,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         calls = self.mock_bridge.br.add_flow.call_args_list
         for call in exp_ingress_classifier, exp_egress_classifier, filter_rule:
             self.assertIn(call, calls)
+        self._assert_invalid_conntrack_entries_deleted(port_dict)
 
     def test_prepare_port_filter_port_security_disabled(self):
         port_dict = {'device': 'port-id',
@@ -630,6 +651,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
                 self.firewall, 'initialize_port_flows') as m_init_flows:
             self.firewall.prepare_port_filter(port_dict)
         self.assertFalse(m_init_flows.called)
+        self.delete_invalid_conntrack_entries_mock.assert_not_called()
 
     def _test_initialize_port_flows_dvr_conntrack_direct(self, network_type):
         port_dict = {
@@ -801,6 +823,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.assertFalse(self.mock_bridge.br.delete_flows.called)
         self.firewall.prepare_port_filter(port_dict)
         self.assertTrue(self.mock_bridge.br.delete_flows.called)
+        self._assert_invalid_conntrack_entries_deleted(port_dict)
 
     def test_update_port_filter(self):
         port_dict = {'device': 'port-id',
@@ -832,6 +855,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             table=ovs_consts.RULES_EGRESS_TABLE)]
         self.mock_bridge.br.add_flow.assert_has_calls(
             filter_rules, any_order=True)
+        self._assert_invalid_conntrack_entries_deleted(port_dict)
 
     def test_update_port_filter_create_new_port_if_not_present(self):
         port_dict = {'device': 'port-id',
@@ -851,15 +875,18 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.assertFalse(self.mock_bridge.br.delete_flows.called)
         self.assertTrue(initialize_port_flows_mock.called)
         self.assertTrue(add_flows_from_rules_mock.called)
+        self._assert_invalid_conntrack_entries_deleted(port_dict)
 
     def test_update_port_filter_port_security_disabled(self):
         port_dict = {'device': 'port-id',
                      'security_groups': [1]}
         self._prepare_security_group()
         self.firewall.prepare_port_filter(port_dict)
+        self.delete_invalid_conntrack_entries_mock.reset_mock()
         port_dict['port_security_enabled'] = False
         self.firewall.update_port_filter(port_dict)
         self.assertTrue(self.mock_bridge.br.delete_flows.called)
+        self.delete_invalid_conntrack_entries_mock.assert_not_called()
 
     def test_update_port_filter_applies_added_flows(self):
         """Check flows are applied right after _set_flows is called."""
@@ -880,6 +907,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.mock_bridge.br.get_vif_port_by_id.return_value = None
         self.firewall.update_port_filter(port_dict)
         self.assertTrue(self.mock_bridge.br.delete_flows.called)
+        self._assert_invalid_conntrack_entries_deleted(port_dict)
 
     def test_remove_port_filter(self):
         port_dict = {'device': 'port-id',
@@ -972,6 +1000,38 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         with testtools.ExpectedException(exceptions.OVSFWPortNotHandled):
             self.firewall._remove_egress_no_port_security('foo')
 
+    def test__initialize_egress_ipv6_icmp(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1],
+            'fixed_ips': ["10.0.0.1"],
+            'allowed_address_pairs': [
+                {'mac_address': 'aa:bb:cc:dd:ee:ff',
+                 'ip_address': '192.168.1.1'},
+                {'mac_address': 'aa:bb:cc:dd:ee:ff',
+                 'ip_address': '2003::1'}
+            ]}
+        of_port = create_ofport(port_dict)
+        self.mock_bridge.br.db_get_val.return_value = {'tag': TESTING_VLAN_TAG}
+        self.firewall._initialize_egress_ipv6_icmp(
+            of_port, set([('aa:bb:cc:dd:ee:ff', '2003::1')]))
+        expected_calls = []
+        for icmp_type in agent_firewall.ICMPV6_ALLOWED_EGRESS_TYPES:
+            expected_calls.append(
+                mock.call(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=95,
+                    in_port=TESTING_VLAN_TAG,
+                    reg5=TESTING_VLAN_TAG,
+                    dl_type='0x86dd',
+                    nw_proto=constants.PROTO_NUM_IPV6_ICMP,
+                    icmp_type=icmp_type,
+                    dl_src='aa:bb:cc:dd:ee:ff',
+                    ipv6_src='2003::1',
+                    actions='resubmit(,%d)' % (
+                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)))
+        self.mock_bridge.br.add_flow.assert_has_calls(expected_calls)
+
     def test_process_trusted_ports_caches_port_id(self):
         vif_port = ovs_lib.VifPort('name', 1, 'id', 'mac', mock.ANY)
         with mock.patch.object(self.firewall.int_br.br, 'get_vifs_by_ids',
