@@ -22,6 +22,7 @@ from neutron_lib import constants as const
 from neutron_lib import context
 from neutron_lib import exceptions
 from neutron_lib.exceptions import l3 as l3_exc
+from neutron_lib.objects import exceptions as o_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as plugin_utils
@@ -394,7 +395,8 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         routers = self.mixin._build_routers_list(self.ctx, routers, gw_ports)
         self.assertIsNone(routers[0].get('gw_port'))
 
-    def _helper_delete_floatingip_agent_gateway_port(self, port_host):
+    def _helper_delete_floatingip_agent_gateway_port(
+            self, port_host, delete_dvr_fip_agent_port_side_effect=None):
         ports = [{
             'id': 'my_port_id',
             portbindings.HOST_ID: 'foo_host',
@@ -410,17 +412,31 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         plugin = mock.Mock()
         directory.add_plugin(plugin_constants.CORE, plugin)
         plugin.get_ports.return_value = ports
-        self.mixin.delete_floatingip_agent_gateway_port(
-            self.ctx, port_host, 'ext_network_id')
+        self.mixin._get_agent_by_type_and_host = mock.Mock(
+            return_value={'id': uuidutils.generate_uuid()})
+
+        with mock.patch.object(
+            router_obj, "DvrFipGatewayPortAgentBinding"
+        ) as dvr_fip_agent_port_obj:
+            dvr_fip_agent_port_obj_instance = (
+                dvr_fip_agent_port_obj.return_value)
+            dvr_fip_agent_port_obj_instance.delete.side_effect = (
+                delete_dvr_fip_agent_port_side_effect)
+
+            self.mixin.delete_floatingip_agent_gateway_port(
+                self.ctx, port_host, 'ext_network_id')
+
         plugin.get_ports.assert_called_with(self.ctx, filters={
             'network_id': ['ext_network_id'],
             'device_owner': [const.DEVICE_OWNER_AGENT_GW]})
         if port_host:
             plugin.ipam.delete_port.assert_called_once_with(
                 self.ctx, 'my_port_id')
+            dvr_fip_agent_port_obj_instance.delete.assert_called_once()
         else:
             plugin.ipam.delete_port.assert_called_with(
                 self.ctx, 'my_new_port_id')
+            dvr_fip_agent_port_obj_instance.delete.assert_called()
 
     def test_delete_floatingip_agent_gateway_port_without_host_id(self):
         self._helper_delete_floatingip_agent_gateway_port(None)
@@ -429,6 +445,16 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         self._helper_delete_floatingip_agent_gateway_port(
             'foo_host')
 
+    def test_delete_floatingip_agent_gateway_port_no_host_id_fip_gw_not_found(
+            self):
+        self._helper_delete_floatingip_agent_gateway_port(
+                None, exceptions.ObjectNotFound(id='my_port_id'))
+
+    def test_delete_floatingip_agent_gateway_port_host_id_fip_gw_not_found(
+            self):
+        self._helper_delete_floatingip_agent_gateway_port(
+            'foo_host', exceptions.ObjectNotFound(id='my_port_id'))
+
     def _setup_delete_current_gw_port_deletes_dvr_internal_ports(
             self, port=None, gw_port=True, new_network_id='ext_net_id_2'):
         router_db = {
@@ -625,6 +651,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         self.assertTrue(create_fip.called)
 
     def test_create_fip_agent_gw_port_if_not_exists_with_l3_agent(self):
+        network_id = _uuid()
         fport_db = {'id': _uuid()}
         self.mixin._get_agent_gw_ports_exist_for_network = mock.Mock(
             return_value=fport_db)
@@ -641,7 +668,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             return_value=fipagent)
         fport = self.mixin.create_fip_agent_gw_port_if_not_exists(
                                                 self.ctx,
-                                                'network_id',
+                                                network_id,
                                                 'host')
         self.assertIsNone(fport)
 
@@ -655,10 +682,84 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 configurations={"agent_mode": "dvr"})
         self.mixin._get_agent_by_type_and_host = mock.Mock(
             return_value=fipagent)
-        fport = self.mixin.create_fip_agent_gw_port_if_not_exists(
-                                                self.ctx,
-                                                'network_id',
-                                                'host')
+        with mock.patch.object(
+            router_obj, "DvrFipGatewayPortAgentBinding",
+        ) as dvr_fip_agent_port_obj:
+            dvr_fip_agent_port_obj_instance = (
+                dvr_fip_agent_port_obj.return_value)
+            fport = self.mixin.create_fip_agent_gw_port_if_not_exists(
+                                                    self.ctx,
+                                                    network_id,
+                                                    'host')
+
+        dvr_fip_agent_port_obj_instance.create.assert_not_called()
+        self.assertIsNotNone(fport)
+        dvr_fip_agent_port_obj_instance.delete.assert_not_called()
+
+    def test_create_fip_agent_gw_port_agent_port_not_created(self):
+        network_id = _uuid()
+        self.mixin._get_agent_gw_ports_exist_for_network = mock.Mock(
+            return_value=None)
+        fipagent = agent_obj.Agent(
+                self.ctx,
+                id=_uuid(),
+                binary='foo-agent',
+                host='host',
+                agent_type='L3 agent',
+                topic='foo_topic',
+                configurations={"agent_mode": "dvr"})
+        self.mixin._get_agent_by_type_and_host = mock.Mock(
+            return_value=fipagent)
+
+        with mock.patch.object(
+            router_obj, "DvrFipGatewayPortAgentBinding",
+        ) as dvr_fip_agent_port_obj,\
+            mock.patch.object(
+                plugin_utils, "create_port", return_value=None):
+
+            dvr_fip_agent_port_obj_instance = (
+                dvr_fip_agent_port_obj.return_value)
+
+            self.assertRaises(
+                exceptions.BadRequest,
+                self.mixin.create_fip_agent_gw_port_if_not_exists,
+                self.ctx, network_id, 'host')
+
+        dvr_fip_agent_port_obj_instance.create.assert_called_once_with()
+        self.mixin._get_agent_gw_ports_exist_for_network.\
+            assert_called_once_with(
+                self.ctx, network_id, 'host', fipagent['id'])
+        dvr_fip_agent_port_obj_instance.delete.assert_called_once_with()
+
+    def test_create_fip_agent_gw_port_if_not_exists_duplicate_port(self):
+        network_id = _uuid()
+        fport_db = {'id': _uuid()}
+        self.mixin._get_agent_gw_ports_exist_for_network = mock.Mock(
+            side_effect=[None, fport_db])
+        fipagent = agent_obj.Agent(
+                self.ctx,
+                id=_uuid(),
+                binary='foo-agent',
+                host='host',
+                agent_type='L3 agent',
+                topic='foo_topic',
+                configurations={"agent_mode": "dvr"})
+        self.mixin._get_agent_by_type_and_host = mock.Mock(
+            return_value=fipagent)
+
+        with mock.patch.object(
+            router_obj.DvrFipGatewayPortAgentBinding, 'create',
+            side_effect=o_exc.NeutronDbObjectDuplicateEntry(
+                mock.Mock(), mock.Mock())
+        ) as dvr_fip_gateway_port_agent_binding_create:
+            fport = self.mixin.create_fip_agent_gw_port_if_not_exists(
+                                                    self.ctx,
+                                                    network_id,
+                                                    'host')
+        dvr_fip_gateway_port_agent_binding_create.assert_called_once_with()
+        self.mixin._get_agent_gw_ports_exist_for_network.assert_has_calls([
+            mock.call(self.ctx, network_id, 'host', fipagent['id']),
+            mock.call(self.ctx, network_id, 'host', fipagent['id'])])
         self.assertIsNotNone(fport)
 
     def test_create_floatingip_agent_gw_port_with_non_dvr_router(self):
@@ -1308,3 +1409,27 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             self.assertRaises(
                 exceptions.BadRequest,
                 self.mixin._get_assoc_data, self.ctx, mock.ANY, mock.Mock())
+
+    def test__delete_dvr_internal_ports(self):
+        payload = mock.Mock()
+        payload.context = mock.Mock()
+        payload.latest_state = {'distributed': True}
+        payload.metadata = {'new_network_id': 'fake-net-1',
+                            'network_id': 'fake-net-2'}
+        plugin = mock.Mock()
+        directory.add_plugin(plugin_constants.CORE, plugin)
+        plugin.get_ports.return_value = []
+        with mock.patch.object(self.mixin,
+                               'delete_floatingip_agent_gateway_port') as \
+            del_port, \
+            mock.patch.object(
+                self.mixin.l3_rpc_notifier,
+                'delete_fipnamespace_for_ext_net') as \
+            del_fip_ns, \
+            mock.patch.object(router_obj.DvrFipGatewayPortAgentBinding,
+                              "delete_objects") as del_binding:
+            self.mixin._delete_dvr_internal_ports(
+                None, None, resources.ROUTER_GATEWAY, payload)
+            del_port.assert_called_once()
+            del_fip_ns.assert_called_once()
+            del_binding.assert_called_once()
