@@ -183,9 +183,8 @@ class TestNovaNotify(base.BaseTestCase):
 
     def test_delete_floatingip_deleted_port_no_notify(self):
         port_id = 'bee50827-bcee-4cc8-91c1-a27b0ce54222'
-        with mock.patch.object(
-            directory.get_plugin(), 'get_port',
-            side_effect=n_exc.PortNotFound(port_id=port_id)):
+        with mock.patch.object(directory.get_plugin(), 'get_port',
+                side_effect=n_exc.PortNotFound(port_id=port_id)):
             returned_obj = {'floatingip':
                             {'port_id': port_id}}
             event = self.nova_notifier.create_port_changed_event(
