@@ -17,6 +17,7 @@ from neutron_lib import fixture
 
 from neutron.plugins.ml2.drivers.agent import capabilities
 from neutron.tests import base
+from neutron.tests import tools
 
 
 class CapabilitiesTest(base.BaseTestCase):
@@ -40,6 +41,6 @@ class CapabilitiesTest(base.BaseTestCase):
         mock_callback = mock.Mock()
         mock_agent_type = mock.Mock()
         capabilities.register(mock_callback, mock_agent_type)
-        self._mgr.subscribe.assert_called_with(mock_callback,
-                                               mock_agent_type,
-                                               events.AFTER_INIT)
+        args = tools.get_subscribe_args(
+            mock_callback, mock_agent_type, events.AFTER_INIT)
+        self._mgr.subscribe.assert_called_with(*args)
